"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const deps_1 = require("./deps");
const path = require("path");
const util = require("util");
const fs = require("./fs");
class StreamOutput {
    constructor(type, stream) {
        this.type = type;
        this.stream = stream;
    }
    static logToFile(msg, logfile) {
        try {
            fs.mkdirpSync(path.dirname(logfile));
            fs.appendFileSync(logfile, deps_1.default.stripAnsi(msg));
        }
        catch (err) {
            console.error(err);
        }
    }
    get output() {
        return this.type === 'stdout' ? deps_1.default.Config.stdout : deps_1.default.Config.stderr;
    }
    write(msg, options = {}) {
        msg = msg || '';
        const log = options.log !== false;
        if (log)
            this.writeLogFile(msg, StreamOutput.startOfLine);
        // conditionally show timestamp if configured to display
        if (StreamOutput.startOfLine && deps_1.default.Config.displayTimestamps) {
            msg = this.timestamp(msg);
        }
        if (deps_1.default.Config.mock) {
            let m = deps_1.default.stripAnsi(msg);
            if (this.type === 'stdout')
                deps_1.default.Config.stdout += m;
            else
                deps_1.default.Config.stderr += m;
        }
        else {
            this.stream.write(msg);
        }
        StreamOutput.startOfLine = msg.endsWith('\n');
    }
    log(data, ...args) {
        let msg = data ? util.format(data, ...args) : '';
        msg += '\n';
        this.write(msg);
    }
    writeLogFile(msg, withTimestamp) {
        if (!this.logfile) {
            return;
        }
        msg = withTimestamp ? this.timestamp(msg) : msg;
        StreamOutput.logToFile(msg, this.logfile);
    }
    get logfile() {
        if (this.type === 'stderr')
            return deps_1.default.Config.errlog;
    }
    timestamp(msg) {
        return `[${deps_1.default.moment().format()}] ${msg}`;
    }
}
StreamOutput.startOfLine = false;
exports.default = StreamOutput;
